<?php
/*-------------------------------------------------
   SEFind/SEDiscover for PHP

   (c) UK 2012 Scannex Electronics Limited
   www.scannex.co.uk   Matt Page
   
	Usage:
   		php sefind.php
	   	php sefind.php NAME/SERIAL
   		php sefind.php NAME/SERIAL DHCP
	 	php sefind.php NAME/SERIAL IP SUBNET GATEWAY
 	
 	e.g.
 		php sefind.php Scannex dhcp
 		php sefind.php 00:02:ae:12:34:56 192.168.0.235 255.255.255.0 192.168.0.1
 	

   2012-05-16:MP: Started project
---------------------------------------------------*/

define("HDAP_PORT", 56024);
define("HDAP_SIGNATURE", "SCANNEX-HDAP\0\0\0\0");
define("HDAP_CMD_NOTHING", 0);
define("HDAP_CMD_QUERY",   1);
define("HDAP_CMD_REPLY",   2);
define("HDAP_CMD_SET",     3);
define("HDAP_CMD_CONFIRM", 4);
define("HDAP_CMD_DENIED",  5);

define("HDAP_OLD_SIZE", 52);	// Length of the old format packets (for NetBuffer)

function bin2mac($bin) {
	return sprintf("%02x:%02x:%02x:%02x:%02x:%02x", 
		ord(substr($bin,0,1)), ord(substr($bin,1,1)), ord(substr($bin,2,1)),
		ord(substr($bin,3,1)), ord(substr($bin,4,1)), ord(substr($bin,5,1))
		);
}

function mac2bin($mac) {
	$mac = preg_replace("/[^A-Fa-f0-9]/","",$mac);
	if (strlen($mac) < 6) $mac = str_pad($mac, 6, "0", STR_PAD_LEFT);
	if (strlen($mac) < 12) $mac = "0002ae".$mac;
	return pack("H12",$mac);
}

function hdap_init_packet() {
	$packet = array();
	$packet["command"] = 0;
	$packet["name"] = "";
	$packet["mac"] = "";
	$packet["dhcp"] = false;
	$packet["ip"] = "0.0.0.0";
	$packet["subnet"] = "255.255.255.255";
	$packet["gateway"] = "0.0.0.0";
	return $packet;
}

function hdap_build_packet($packet) {
	if ($packet["dhcp"] == false) $dhcp = 0; else $dhcp = 0xff;
	$data = pack("a16Ca16a6CNNNNNN",
		HDAP_SIGNATURE,
		$packet["command"],
		$packet["name"],
		mac2bin($packet["mac"]),
		$dhcp,
		ip2long($packet["ip"]),
		ip2long($packet["subnet"]),
		ip2long($packet["gateway"]),
		3,
		0,
		0);
	return $data;
}

function hdap_unpack($data) {
	if (!$data or $data=="") return nil;
	if (strlen($data)==HDAP_OLD_SIZE) {
		$array = unpack("a16signature/Ccommand/a16name/a6mac/Cdhcp/Nip/Nsubnet/Ngateway", $data);
		$array["version"] = 1;
		$array["remoteip"] = 0;
		$array["interfaceid"] = 0;
	}
	else
		$array = unpack("a16signature/Ccommand/a16name/a6mac/Cdhcp/Nip/Nsubnet/Ngateway/Nversion/Nremoteip/Ninterfaceid", $data);			
	$array["ip"] = long2ip($array["ip"]);
	$array["gateway"] = long2ip($array["gateway"]);
	$array["subnet"] = long2ip($array["subnet"]);
	$array["mac"] = bin2mac($array["mac"]);
	$array["dhcp"] = ($array["dhcp"] != chr(0));
	return $array;
}

function hdap_discover($sock) {
	$packet = hdap_init_packet();
	$packet["command"] = HDAP_CMD_QUERY;
	$buf = hdap_build_packet($packet);
	socket_sendto($sock, $buf, strlen($buf), 0, "255.255.255.255", HDAP_PORT);	
}

function hdap_find($sock) {
	$result = array();
	socket_set_option($sock, SOL_SOCKET, SO_RCVTIMEO, array("sec"=>10, "usec"=>0));
	while (($len = @socket_recvfrom($sock, $data, 2048, 0, $cIP, $cPort)) != false) {
		$packet = hdap_unpack($data);
		if ($packet["command"] == HDAP_CMD_REPLY) {
			$result[$packet["mac"]] = $packet;
		}
		socket_set_option($sock, SOL_SOCKET, SO_RCVTIMEO, array("sec"=>0, "usec"=>250000));
	}
	ksort($result);
	return $result;
}

function hdap_change($sock, &$packet) {
	$packet["command"] = HDAP_CMD_SET;
	$buf = hdap_build_packet($packet);
	socket_sendto($sock, $buf, strlen($buf), 0, "255.255.255.255", HDAP_PORT);
	while (($len = @socket_recvfrom($sock, $data, 2048, 0, $cIP, $cPort)) != false) {
		$packet = hdap_unpack($data);
		switch ($packet["command"]) {
			case HDAP_CMD_CONFIRM : return true;
			case HDAP_CMD_DENIED : return false;
			default: ;
		}
		if ($packet["command"] == HDAP_CMD_REPLY) {
			$result[$packet["mac"]] = $packet;
		}
	}	
	return NULL;
}

function find_in_list($list, $value) {
	foreach($list as $key => $v) {
		if (mac2bin($key) == mac2bin($value)) {
			return $key;
		}
		if (strcasecmp($v["name"], $value) == 0) {
			return $key;
		}
	}
	return NULL;
}

function show_entry($val) {
	if ($val["dhcp"]) $dhcp = "DHCP"; else $dhcp = "";
	echo sprintf("%s  %-16s %-4s %-15s  %-15s  %-15s\n",
			$val["mac"], 
			$val["name"], 
			$dhcp,
			$val["ip"],
			$val["subnet"],
			$val["gateway"]
			);
}

function show_list($list, $akey) {
	echo "Serial Number      Name             DHCP IP Address       Subnet           Gateway\n";
	echo "----------------   ---------------- ---- ---------------  ---------------  ---------------\n";
	foreach ($list as $key => $val) {
		if (!$akey or ($key == $akey)) {
			show_entry($val);
		}
	}
	echo "\n";
}

// Open up the UDP socket needed
$sock = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
$result = socket_bind($sock, "0.0.0.0", HDAP_PORT) or die("!! Could not open SEDiscover port\n");
socket_set_option($sock, SOL_SOCKET, SO_BROADCAST, 1);
socket_set_block($sock);

hdap_discover($sock);		// Send out the discovery packet
$list = hdap_find($sock);	// Get an array of all the found buffers

if ($argc < 2) {
	// Now display the list of found buffers
	show_list($list, NULL);
} else {
	// User is asking to find a buffer, and possibly change its values
	$mac = find_in_list($list, $argv[1]);
	if (!$mac) die("!! Could not find $argv[1] in the list!\n");
	$val = $list[$mac];
	$name = $val["name"];
	echo "FOUND: ";
	show_entry($val);
	if ($argc > 2) {
		if ((strcasecmp($argv[2], "dhcp") == 0) or (ip2long($argv[2]) == 0)) {
			$val["dhcp"] = true;
		} else {
			$val["ip"] = $argv[2];
			if (@$argv[3]) $val["subnet"] = $argv[3];
			if (@$argv[4]) $val["gateway"] = $argv[4];
			$val["dhcp"] = false;
		}
		if (hdap_change($sock, $val)) {
			echo "Updated\n";
		} else {
			echo "!! CHANGES REFUSED !!\n";
		}
	}
	echo "\n";
}
socket_close($sock);
?>
